@@ -1,9 +1,9 @@
 /*****************************************************************************
- * RRDtool 1.2.11  Copyright by Tobi Oetiker, 1997-2005
+ * RRDtool 1.2.15  Copyright by Tobi Oetiker, 1997-2006
  *****************************************************************************
  * rrd_error.c   Common Header File
  *****************************************************************************
- * $Id: rrd_error.c 663 2005-07-25 14:12:58Z oetiker $
+ * $Id: rrd_error.c 860 2006-07-14 12:11:16Z oetiker $
  * $Log$
  * Revision 1.4  2003/02/22 21:57:03  oetiker
  * a patch to avoid a memory leak and a Makefile.am patch to
@@ -111,8 +111,8 @@ rrd_new_context(void) {
 
     if (rrd_ctx) {
 	rrd_ctx->len = 0;
-	rrd_ctx->rrd_error = malloc(MAXLEN);
-	rrd_ctx->lib_errstr = malloc(ERRBUFLEN);
+	rrd_ctx->rrd_error = malloc(MAXLEN+10);
+	rrd_ctx->lib_errstr = malloc(ERRBUFLEN+10);
 	if (rrd_ctx->rrd_error && rrd_ctx->lib_errstr) {
 	    *rrd_ctx->rrd_error = 0;
 	    *rrd_ctx->lib_errstr = 0;
