@@ -1,5 +1,5 @@
 /*****************************************************************************
- * RRDtool 1.0.50  Copyright Tobias Oetiker, 1997 - 2005
+ * RRDtool 1.2.15  Copyright by Tobi Oetiker, 1997-2006
  *****************************************************************************
  * rrd_first Return
  *****************************************************************************
@@ -8,16 +8,13 @@
 
 #include "rrd_tool.h"
 
+
 time_t
-rrd_first(int argc, char **argv)    
-{   
-    long rra_start,
-         timer;
-    unsigned long target_rraindex=0;
-    time_t then;
+rrd_first(int argc, char **argv)
+{
+    int target_rraindex=0;
     char *endptr;
-    FILE *in_file;
-    rrd_t rrd;
+    optind = 0; opterr = 0;  /* initialize getopt */
 
     while (1){
         static struct option long_options[] =
@@ -29,7 +26,7 @@ rrd_first(int argc, char **argv)
         int opt;
         opt = getopt_long(argc, argv, "", long_options, &option_index);
 
-        if (opt == EOF)
+        if(opt == EOF)
             break;
 
         switch(opt) {
@@ -40,23 +37,36 @@ rrd_first(int argc, char **argv)
                 return(-1);
             }
             break;
-          case '?':
-            rrd_set_error("unknown option '-%c'",optopt);
+          default:
+            rrd_set_error("usage rrdtool %s [--rraindex number] file.rrd", argv[0]);
             return(-1);
         }
     }
 
-    if (optind >= argc){
+    if(optind >= argc){
         rrd_set_error("not enough arguments");
         return -1;       
     }     
- 
-    if(rrd_open(argv[optind],&in_file,&rrd, RRD_READONLY)==-1){
+
+    return(rrd_first_r(argv[optind], target_rraindex));
+}
+
+
+time_t
+rrd_first_r(const char *filename, const int rraindex)
+{
+    long rra_start,
+         timer;
+    time_t then;
+    FILE *in_file;
+    rrd_t rrd;
+
+    if(rrd_open(filename,&in_file,&rrd, RRD_READONLY)==-1){
         rrd_set_error("could not open RRD");
         return(-1);
     }
 
-    if((target_rraindex < 0) || (target_rraindex >= rrd.stat_head->rra_cnt)) {
+    if((rraindex < 0) || (rraindex >= (int)rrd.stat_head->rra_cnt)) {
         rrd_set_error("invalid rraindex number");
         return(-1);
     }
@@ -64,19 +74,19 @@ rrd_first(int argc, char **argv)
     rra_start = ftell(in_file);    
     fseek(in_file,
           (rra_start +
-           (rrd.rra_ptr[target_rraindex].cur_row+1) *
+           (rrd.rra_ptr[rraindex].cur_row+1) *
            rrd.stat_head->ds_cnt *
            sizeof(rrd_value_t)),
           SEEK_SET);
-    timer = - (rrd.rra_def[target_rraindex].row_cnt-1);
-    if (rrd.rra_ptr[target_rraindex].cur_row + 1 > rrd.rra_def[target_rraindex].row_cnt) {
+    timer = - (rrd.rra_def[rraindex].row_cnt-1);
+    if (rrd.rra_ptr[rraindex].cur_row + 1 > rrd.rra_def[rraindex].row_cnt) {
       fseek(in_file,rra_start,SEEK_SET);
     }
     then = (rrd.live_head->last_up -
             rrd.live_head->last_up %
-            (rrd.rra_def[target_rraindex].pdp_cnt*rrd.stat_head->pdp_step)) +
+            (rrd.rra_def[rraindex].pdp_cnt*rrd.stat_head->pdp_step)) +
            (timer * 
-            rrd.rra_def[target_rraindex].pdp_cnt*rrd.stat_head->pdp_step);
+            rrd.rra_def[rraindex].pdp_cnt*rrd.stat_head->pdp_step);
 
     rrd_free(&rrd);
     fclose(in_file);
