@@ -1,13 +1,7 @@
 /*****************************************************************************
- * RRDtool 1.0.50  Copyright Tobias Oetiker, 1997 - 2000
+ * RRDtool 1.2.15  Copyright by Tobi Oetiker, 1997-2006
  *****************************************************************************
  * rrd_tool.h   Common Header File
- *****************************************************************************
- * $Id: rrd_tool.h,v 1.1.1.1 2002/02/26 10:21:37 oetiker Exp $
- * $Log: rrd_tool.h,v $
- * Revision 1.1.1.1  2002/02/26 10:21:37  oetiker
- * Intial Import
- *
  *****************************************************************************/
 #ifdef  __cplusplus
 extern "C" {
@@ -17,12 +11,11 @@ extern "C" {
 #ifndef _RRD_TOOL_H
 #define _RRD_TOOL_H
 
-#ifdef WIN32
-# include "ntconfig.h"
+#if defined(_WIN32) && !defined(__CYGWIN__) && !defined(__CYGWIN32__) && !defined(HAVE_CONFIG_H)
+#include "../confignt/config.h"
 #else
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-#include "config_aux.h"
 #endif
 #endif
 
@@ -52,6 +45,36 @@ extern "C" {
 #if HAVE_MATH_H
 # include <math.h>
 #endif
+/* Sorry: don't know autoconf as well how to check the exist of
+   dirent.h ans sys/stat.h
+*/
+
+#if HAVE_DIRENT_H
+# include <dirent.h>
+# define NAMLEN(dirent) strlen((dirent)->d_name)
+#else
+# define dirent direct
+# define NAMLEN(dirent) (dirent)->d_namlen
+# if HAVE_SYS_NDIR_H
+#  include <sys/ndir.h>
+# endif
+# if HAVE_SYS_DIR_H
+#  include <sys/dir.h>
+# endif
+# if HAVE_NDIR_H
+#  include <ndir.h>
+# endif
+#endif
+
+#if HAVE_SYS_TYPES_H
+# include <sys/types.h>
+#endif
+
+#if HAVE_SYS_STAT_H
+# include <sys/stat.h>
+#endif
+
+
 #if HAVE_UNISTD_H
 # include <unistd.h>
 #endif
@@ -72,21 +95,30 @@ extern int getrusage(int, struct rusage *);
 
 #include "rrd.h"
 
-#ifndef WIN32
+#if defined(_WIN32) && !defined(__CYGWIN__) && !defined(__CYGWIN32__)
+
+/* Win32 only includes */
+
+#include <float.h>        /* for _isnan  */
+#include <io.h>           /* for chdir   */
+
+#ifndef __MINGW32__
+#define isnan _isnan
+#define finite _finite
+#define isinf(a) (_fpclass(a) == _FPCLASS_NINF || _fpclass(a) == _FPCLASS_PINF)
+#endif
+struct tm* localtime_r(const time_t *timep, struct tm* result);
+char* ctime_r(const time_t *timep, char* result);
+struct tm* gmtime_r(const time_t *timep, struct tm* result);
+char *strtok_r(char *str, const char *sep, char **last);
+
+#else
 
 /* unix-only includes */
 #ifndef isnan
 int isnan(double value);
 #endif
 
-#else
-
-/* Win32 only includes */
-
-#include <float.h>        /* for _isnan  */
-#define isnan _isnan
-#define finite _finite
-#define isinf(a) (_fpclass(a) == _FPCLASS_NINF || _fpclass(a) == _FPCLASS_PINF)
 #endif
 
 /* local include files -- need to be after the system ones */
@@ -106,12 +138,14 @@ int isnan(double value);
 /* rrd info interface */
 enum info_type   { RD_I_VAL=0,
 	       RD_I_CNT,
-	       RD_I_STR  };
+	       RD_I_STR, 
+		   RD_I_INT };
 
 typedef union infoval { 
     unsigned long u_cnt; 
     rrd_value_t   u_val;
     char         *u_str;
+    int		  u_int;
 } infoval;
 
 typedef struct info_t {
@@ -121,16 +155,14 @@ typedef struct info_t {
     struct info_t   *next;
 } info_t;
 
-
 info_t *rrd_info(int, char **);
+info_t *rrd_update_v(int, char **);
+char * sprintf_alloc(char *, ...);
+info_t *info_push(info_t *, char *, enum info_type, infoval);
 
 /* HELPER FUNCTIONS */
-int GifSize(FILE *, long *, long *);
-int PngSize(FILE *, long *, long *);
-int PngSize(FILE *, long *, long *);
 
-#include <gd.h>
-void gdImagePng(gdImagePtr im, FILE *out);
+int PngSize(FILE *, long *, long *);
 
 int rrd_create_fn(char *file_name, rrd_t *rrd);
 int rrd_fetch_fn(char *filename, enum cf_en cf_idx,
@@ -141,10 +173,11 @@ int rrd_fetch_fn(char *filename, enum cf_en cf_idx,
 		 rrd_value_t **data);
 
 void rrd_free(rrd_t *rrd);
+void rrd_freemem(void *mem);
 void rrd_init(rrd_t *rrd);
 
-int rrd_open(char *file_name, FILE **in_file, rrd_t *rrd, int rdwr);
-int readfile(char *file, char **buffer, int skipfirst);
+int rrd_open(const char *file_name, FILE **in_file, rrd_t *rrd, int rdwr);
+int readfile(const char *file, char **buffer, int skipfirst);
 
 #define RRD_READONLY    0
 #define RRD_READWRITE   1
@@ -154,11 +187,13 @@ enum dst_en dst_conv(char *string);
 long ds_match(rrd_t *rrd,char *ds_nam);
 double rrd_diff(char *a, char *b);
 
-#endif
+    /* rrd_strerror is thread safe, but still it uses a global buffer
+       (but one per thread), thus subsequent calls within a single
+       thread overwrite the same buffer */
+const char *rrd_strerror(int err);
 
+#endif
 
 #ifdef  __cplusplus
 }
 #endif
-
-
